@@ -6,6 +6,7 @@ in the source distribution for its full text.
 */
 
 #include "Hashtable.h"
+#include "XAlloc.h"
 
 #include <stdlib.h>
 #include <assert.h>
@@ -62,8 +63,8 @@ int Hashtable_count(Hashtable* this) {
 
 static HashtableItem* HashtableItem_new(unsigned int key, void* value) {
    HashtableItem* this;
-   
-   this = (HashtableItem*) malloc(sizeof(HashtableItem));
+
+   this = xMalloc(sizeof(HashtableItem));
    this->key = key;
    this->value = value;
    this->next = NULL;
@@ -72,11 +73,11 @@ static HashtableItem* HashtableItem_new(unsigned int key, void* value) {
 
 Hashtable* Hashtable_new(int size, bool owner) {
    Hashtable* this;
-   
-   this = (Hashtable*) malloc(sizeof(Hashtable));
+
+   this = xMalloc(sizeof(Hashtable));
    this->items = 0;
    this->size = size;
-   this->buckets = (HashtableItem**) calloc(size, sizeof(HashtableItem*));
+   this->buckets = (HashtableItem**) xCalloc(size, sizeof(HashtableItem*));
    this->owner = owner;
    assert(Hashtable_isConsistent(this));
    return this;
@@ -118,10 +119,10 @@ void Hashtable_put(Hashtable* this, unsigned int key, void* value) {
 
 void* Hashtable_remove(Hashtable* this, unsigned int key) {
    unsigned int index = key % this->size;
-   
+
    assert(Hashtable_isConsistent(this));
 
-   HashtableItem** bucket; 
+   HashtableItem** bucket;
    for (bucket = &(this->buckets[index]); *bucket; bucket = &((*bucket)->next) ) {
       if ((*bucket)->key == key) {
          void* value = (*bucket)->value;
